C *****************************************************************************
C      EMPFT SUBROUTINES
C *****************************************************************************
C
C      GLOBAL
C      GLOBAL_CC
C      REFINE
C      REFINE_CC
C
C      ADJUST_NVIEW_MOD
C      CALC_MOD_TPS
C      CALC_TP_MASK
C      GET_BESTMAG
C      GET_BESTPRJ
C      GET_ITHE2
C      GET_PRJS_G
C      GET_PFTS_G
C      GET_PRJSPFTS_R
C      LIST_CCS
C      LOADSTATS_R
C      PFILE_INFO
C      SAVE_PRJS
C
C      PFTCC


C  *************************************************************************
C  *                                                         *
C  *             *****     SUBROUTINE ADJUST_NVIEW_MOD     *****           *
C  *                                                         *
C  *  If THETA = 0 or 180 then PHI is undetermined and there are (NPHI-1)  *
C  *  fewer possible orientations.  Adjust NVIEW_MOD accordingly.  It will *
C  *  be changed back to the NTHE*NPHI value after the analysis is over.   *
C  *  (This reversion will occur in the main program.)  Used only in       *
C  *  REFINE mode.                                                         *
C  *                                                         *
C  *  D. Belnap, 30 Jan 1997, NIH                                 *
C  *************************************************************************
C
      SUBROUTINE ADJUST_NVIEW_MOD (THETA1, THETA2, DEL_THE, 
     1  NVIEW_MOD, NPHI)
C
      IMPLICIT NONE
C
      INTEGER*4 NVIEW_MOD, NPHI
      REAL*4    THE, THETA1, THETA2, DEL_THE
C ----------------------------------------------------------------------------
C
      THE = THETA1
      DO WHILE (THE.LE.THETA2)
        IF (THE.EQ.0.0 .OR. THE.EQ.180.0) THEN
          NVIEW_MOD = NVIEW_MOD - NPHI + 1
        ENDIF
        THE = THE + DEL_THE
      ENDDO
C
      END



C  ****************************************************************************
C  *                                                                          *
C  *                 *****     SUBROUTINE CALC_TP_MASK     *****              *
C  *                                                                          *
C  *    "MASK" ARRAY SERVES TWO PURPOSES:                                     *
C  *      1. TALLY # MODEL PROJECTIONS NEEDED AT EACH THE,PHI GRID POINT.     *
C  *      2. SIGNIFY AN ID FOR EACH UNIQUE PROJECTION THAT THEN IDENTIFIES    *
C  *         THE LOCATION OF APPROPRIATE DATA IN THE PFTS AND PRJS ARRAYS.    *
C  *                                                                          *
C  *    APRIL 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.          *
C  *                                                                          *
C  *    LAST MODIFIED 28-APR-1998 BY T. BAKER (Implement new code).           *
C  ****************************************************************************
C
      SUBROUTINE CALC_TP_MASK (NTHE, NPHI, NIDS_TOT, TS, PS, MOD_TS, 
     1 MOD_PS, MASK, DPHIS, THE1, THE2, DTHE, DPHI_90, NUNIQUE)
C
      IMPLICIT NONE
C
      INTEGER*4 NTHE, NPHI, NIDS_TOT, ITH, IPH, ITHE, IPHI, ID
      INTEGER*4 I, J, ITHETA, NUM, NUNIQUE, NNPHI, MASK(NPHI,NTHE,2)
      REAL*4    THETA, PHI, MOD_TS(NTHE), MOD_PS(NPHI,NTHE)
      REAL*4    PHT_PHIMAX, TS(NIDS_TOT), PS(NIDS_TOT), DPHIS(NTHE)
      REAL*4    DELTA, THE1, THE2, DTHE, DPHI_90, DPHI, SPEED
      LOGICAL   SIGN
      real*8    deg_to_rad
C  ----------------------------------------------------------------------------
C
      deg_to_rad = acos(-1.0d0)/180.0d0
      NUM     = 0
      NUNIQUE = 0
C  ***************************
C  *    INITIALIZE "MASK"    *
C  ***************************
      DO J = 1,NTHE
        DO I = 1,NPHI
          MASK (I,J,1) = 0
          MASK (I,J,2) = 0
        ENDDO
      ENDDO
C  **********************************************
C  *    FILL UP DPHIS, MOD_TS, MOD_PS ARRAYS    *
C  **********************************************
      DO J = 1,NTHE
        THETA = THE1 + (J-1)*DTHE
        MOD_TS(J) = THETA
        IF(THETA.EQ.0.0 .OR. THETA.EQ.180.0) THEN
           DPHIS(J) = 0.0
         ELSE
           DPHIS(J) = DPHI_90/SIN(deg_to_rad*THETA)
        ENDIF
        DO I = 1,NPHI
          MOD_PS(I,J) = (I-1)*DPHIS(J)
        ENDDO
      ENDDO
C  *************************************************************
C  *    RESET TS,PS ARRAYS SO T,P VALUES LIE AT GRID POINTS    *
C  *************************************************************
      DO ID = 1, NIDS_TOT
        SIGN   = .TRUE.
        ITHETA = NINT(TS(ID))
        DELTA  = TS(ID) - ITHETA
        TS(ID) = ITHETA + NINT(DELTA/DTHE)*DTHE
        DPHI   = DPHI_90/SIN(deg_to_rad*TS(ID))
        IF(PS(ID).LT.0.0) SIGN = .FALSE.
        PS(ID) = NINT(ABS(PS(ID))/DPHI) * DPHI
        IF(.NOT.SIGN) PS(ID) = -PS(ID)
      ENDDO
C  ********************************************************
C  *    MAP THE "WINDOW" FOR EACH PARTICLE INTO "MASK"    *
C  ********************************************************
      DO ID = 1, NIDS_TOT
        ITH = NINT( ((TS(ID)-THE1)/DTHE) ) + 1
        DO 300 ITHE = ITH-3,ITH+3
          IF(ITHE.GE.1 .AND. ITHE.LE.NTHE) THEN
            IPH = NINT( ABS(PS(ID))/DPHIS(ITHE) ) + 1
            DO 200 IPHI = IPH-3,IPH+3
              IF(IPHI.GE.1 .AND. IPHI.LE.NPHI) THEN
                MASK(IPHI,ITHE,1) = MASK(IPHI,ITHE,1) + 1
                NUM               = NUM + 1
                IF(MASK(IPHI,ITHE,1).EQ.1) NUNIQUE = NUNIQUE + 1
              ENDIF
  200            ENDDO                         ! LOOP OVER PHI GRID POINTS
          ENDIF
  300        ENDDO                               ! LOOP OVER THETA GRID POINTS

ccc      call pif_write_debugi4 ('mask.pif', mask, nphi, nthe)

      ENDDO                               ! LOOP OVER ALL PARTICLES
C  ***************************************
C  *    VT SCREEN OUTPUT FOR DEBUGGING   *
C  ***************************************
ccc      nnphi = 66
ccc      if(nphi.lt.66) nnphi = nphi
ccc      DO I = 1,NTHE
ccc        WRITE(*,1000) (MASK(J,I,1),J=1,NNPHI)
ccc      ENDDO
C
      PRINT '(I8,A)', NUM, ' total number of projections needed'
      PRINT '(I8,A)', NUNIQUE, ' of these are unique'
      SPEED = FLOAT(NUM)/FLOAT(NUNIQUE)
      PRINT '(A,F5.2,A)', ' Potential compute savings:', SPEED, ' times'
C  ******************************************************************
C  *    FILL MASK(IP,IT,2) WITH POINTERS TO PFTS AND PRJS ARRAYS    *
C  ******************************************************************
      NUM = 0
      DO J = 1,NTHE
        DO I = 1,NPHI
          IF(MASK(I,J,1).NE.0) THEN
            NUM         = NUM + 1
            MASK(I,J,2) = NUM
          ENDIF
        ENDDO
      ENDDO
C
      RETURN
 1000      FORMAT(1X,66I2)
      END


C  *****************************************************************************
C  *                                                                           *
C  *                *****     SUBROUTINE GET_BESTPRJ       *****               *
C  *                                                                           *
C  *    READ IN THE APPROPRIATE PROJECTION FOR CROSS-CORRELATION WITH THE      *
C  *     RAW IMAGE.                                                            *
C  *                                                                           *
C  *    ADDITIONAL TEST FOR DO-LOOP: NUM.LE.NVIEW_MOD (KAD)                    *
C  *                                                                           *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, T. BAKER.                        *
C  *                                                                           *
C  *    MODIFIED 23-DEC-1997 BY T. BAKER (Using PRJS in memory).               *
C  *    MODIFIED 14-JAN-1998 BY TSB/RWA (*.PIF input of PFTs/PRJs).            *
C  *    MODIFIED  6-FEB-1998 BY TSB (Use of INDEX instead of T/P arrays).      *
C  *    MODIFIED 26-FEB-1998 BY TSB (Make UNIX compatible).                    *
C  *    LAST MODIFIED 29-APR-1998 BY TSB (Renamed from PFTCC_GETPRJ).          *
C  *****************************************************************************
C
      SUBROUTINE GET_BESTPRJ (BESTPRJ, NCOLBIN, NROWBIN, NVIEWS,
     1 PRJS, MODE, INDEX, PRJHEAD)
C
      IMPLICIT NONE
C
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T) PRJHEAD
C
      INTEGER*4 NCOLBIN, NROWBIN, I, J, NUM, NVIEWS, MODE
      INTEGER*4 INDEX
      REAL*4    BESTPRJ(NCOLBIN,NROWBIN), PRJS(NCOLBIN,NROWBIN,NVIEWS)
C  ----------------------------------------------------------------------------
C
      IF(MODE.LT.0) GO TO 100
C  *************************************************************************
C  *    HERE TO READ IN THE BEST MODEL PROJECTION STORED IN CORE MEMORY    *
C  *************************************************************************
      DO J = 1,NROWBIN
        DO I = 1,NCOLBIN
          BESTPRJ(I,J) = PRJS(I,J,INDEX)
        ENDDO
      ENDDO
      RETURN
C  **********************************************************************
C  *    HERE IF PRJ DATA STORED ON DISK: Just read the section needed   *
C  **********************************************************************
 100  CALL PIF_READ_SECI4 (PRJHEAD, BESTPRJ, NCOLBIN, NROWBIN, INDEX)
C
      RETURN
      END

C  ****************************************************************************
C  *                                                                          *
C  *                   *****     SUBROUTINE GET_ITHE2     *****               *
C  *                                                                          *
C  *    DETERMINE THE THETA LIMIT FOR THE NEXT WINDOW OF MEMORY.              *
C  *                                                                          *
C  *    MAY 1998 PURDUE UNIVERSITY AlphaStation 500/400, T. BAKER.            *
C  ****************************************************************************
C
      SUBROUTINE GET_ITHE2 (N_INCORE, NCOLBIN, NROWBIN, NROT, NANNULI, 
     1 NTHE, NPHI, ITHE1, ITHE2, MASK, PRJS, PFTS, SLIDE, OFFSET)
C
      IMPLICIT NONE
C
      INTEGER*4 N_INCORE, ITHE1, ITHE2, NTHE, NPHI, I, J, NUM_THIS_THE
      INTEGER*4 MASK(NPHI,NTHE,2), ID_OLD, ID_NEW, IPHI, ITHE
      INTEGER*4 NROT_BY_4, NA, NR, COL, ROW, ITEST
      INTEGER*4 NCOLBIN, NROWBIN, NANNULI, NROT, OFFSET, ITHE_REUSE
      REAL*4    PRJS(NCOLBIN,NROWBIN,N_INCORE)
      REAL*4    PFTS((NROT/4),NANNULI,N_INCORE)
      LOGICAL   SLIDE
C  ----------------------------------------------------------------------------
C
C  **********************************************
C  *    FIND PRJ/PFT DATA THAT CAN BE REUSED    *
C  **********************************************
      DO ITHE = ITHE1,ITHE2
        DO IPHI = 1,NPHI
          IF(MASK(IPHI,ITHE,1).GE.1) THEN             ! FOUND REUSABLE DATA
            ITHE_REUSE = ITHE
            GO TO 100
          ENDIF
        ENDDO
      ENDDO
C  ****************************************************************************
C  *    HERE IF NO DATA OVERLAP AND HENCE NO NEED TO COPY TO TOP OF ARRAYS    *
C  ****************************************************************************
      ITHE1  = ITHE2 + 1
      OFFSET = 0
      GO TO 200
C  *****************************************************************
C  *    MOVE THE REUSABLE PRJ/PFT DATA TO THE TOP OF THE ARRAYS    *
C  *****************************************************************
  100      NROT_BY_4 = NROT/4
      ITHE1     = ITHE_REUSE
      ID_NEW    = 0
C
      DO ITHE = ITHE1,ITHE2
        DO IPHI = 1,NPHI
          IF(MASK(IPHI,ITHE,1).GE.1) THEN
            ID_OLD            = MASK(IPHI,ITHE,2)
            ID_NEW            = ID_NEW + 1
            DO ROW = 1,NROWBIN
              DO COL = 1,NCOLBIN
                PRJS(COL,ROW,ID_NEW) = PRJS(COL,ROW,ID_OLD)
              ENDDO
            ENDDO
            DO NA = 1,NANNULI
              DO NR = 1,NROT_BY_4
                PFTS(NR,NA,ID_NEW) = PFTS(NR,NA,ID_OLD)
              ENDDO
            ENDDO
          ENDIF
        ENDDO
      ENDDO
      OFFSET   = ID_NEW
C
  200      IF(ITHE1.GT.NTHE) THEN
        SLIDE = .FALSE.
        RETURN
      ENDIF
C       ******************************************************************
C       *    DETERMINE NEW UPPER LIMIT ON THETA FOR NEXT WINDOW.         *
C-------*    SEE HOW MANY VIEWS CAN BE PROCESSED - RESTRICTION IS THAT   *
C       *     ONLY FULL ROWS (SAME THETA VALUE) ARE PROCESSED.           *
C       ******************************************************************
      ITEST = 0
      ITHE2 = NTHE
C
      DO ITHE = ITHE1,NTHE
        NUM_THIS_THE = 0
        DO IPHI = 1,NPHI
          IF(MASK(IPHI,ITHE,1).GE.1) THEN
            ITEST             = ITEST + 1
            MASK(IPHI,ITHE,2) = ITEST
            NUM_THIS_THE      = NUM_THIS_THE + 1
          ENDIF
          IF(ITEST.GT.N_INCORE) THEN
            ITEST = ITEST - NUM_THIS_THE
            IF(ITEST-OFFSET.LE.0) THEN
              PRINT *, '********************************************'
              PRINT *, 'GET_ITHE2: Can not process another THETA row'
              PRINT *, 'STOP PROGRAM                                '
              CALL TSBEND
            ENDIF
            ITHE2 = ITHE - 1
            GO TO 300
          ENDIF
        ENDDO                                  ! DO IPHI = 1,NPHI
      ENDDO                                  ! DO ITHE = ITHE_NEW,NTHE
C
  300      RETURN
      END


C  *****************************************************************************
C  *                                                                           *
C  *                 *****     SUBROUTINE LOADSTATS_R     *****                *
C  *                                                                           *
C  *    READ IN POLAR FFTS FOR THE SEARCH WINDOW TO SET UP PFT_AVGS AND SQ2    *
C  *     ARRAYS.                                                               *
C  *                                                                           *
C  *    DECEMBER 1987  PURDUE UNIVERSITY VAX 11/750, T. BAKER                  *
C  *    MAY 1992  PURDUE UNIVERSITY VAX 8550, J. TESMER.                       *
C  *                                                                           *
C  *    Moved THETA,PHI calculations to CALC_MOD_TPS, 30 Jan 1997, D. Belnap   *
C  *    MODIFIED 23-DEC-1997 BY T. BAKER (Using memory for PFTS).              *
C  *    MODIFIED 14-JAN-1998 BY TSB/RWA (*.PIF input of PFTs/PRJs).            *
C  *    MODIFIED  6-MAR-1998 BY TSB (Make UNIX compatible).                    *
C  *    MODIFIED 29-APR-1998 BY TSB (Adapted from PFTCC_FILL).                 *
C  *    LAST MODIFIED  4-MAY-1998 BY TSB (Implement sliding window).           *
C  *****************************************************************************
C
      SUBROUTINE LOADSTATS_R (N_INCORE, PFTS, PFT_AVGS, NROT, NANNULI, 
     1 SQ2, JCUT, PIXRES_MIN, PIXRES_MAX, ANN_LO, ANN_HI, VAR, RMS, 
     2 AVER, VAROPT, SIGCUT, TMP, MASK, NTHE, NPHI, THETA, PHI, 
     3 MOD_TS, DTHE, DPHIS, NVIEW_THISIMG)
C
      IMPLICIT NONE
C
      include '../include/pifhead.inc'
      TYPE(PIFHEAD_T)  MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD
      COMMON /PIFHEAD/ MAPHEAD, IMGHEAD, PRJHEAD, PFTHEAD, SFHEAD

      INTEGER*4 NROT, NANNULI, NVIEW_THISIMG, I, J, JN, JCUT, NROT_BY_4
      INTEGER*4 ANN_LO, ANN_HI, IR, R, NUM, VAR((NROT/4),NANNULI)
      INTEGER*4 N_INCORE, NTHE, NPHI, MASK(NPHI,NTHE,2), MODEL_ID
      INTEGER*4 ITH, IPH, ITHE, IPHI, ITHE1, IPHI1, ITHE2, IPHI2
      REAL*4    RMS((NROT/4),NANNULI), AVER((NROT/4),NANNULI)
      REAL*4    TMP((NROT/4),NANNULI), PFTS((NROT/4),NANNULI,N_INCORE)
      REAL*4    PI, RADIAN, DEGREE, TMP2, SIGCUT, FACTOR, TWOPIR
      REAL*4    PFT_AVGS(N_INCORE), SQ2(N_INCORE), PTS, AVE, AVE2
      REAL*4    PIXRES, PIXRES_MIN, PIXRES_MAX, THETA, PHI, DTHE
      REAL*4    MOD_TS(NTHE), DPHIS(NTHE)
      LOGICAL   VAROPT
C  ----------------------------------------------------------------------------
C
      CALL PIRADDEG (PI, RADIAN, DEGREE)
      NROT_BY_4  = NROT/4
C  *****************************************************
C  *    COUNT UP THE VIEWS AVAILABLE FOR THIS IMAGE    *
C  *****************************************************
      NVIEW_THISIMG = 0
      ITH           = NINT ( (THETA-MOD_TS(1))/DTHE ) + 1
      IPH           = NINT (ABS(PHI)/DPHIS(ITH)) + 1
      ITHE1         = ITH - 3
      ITHE2         = ITH + 3
      IPHI1         = IPH - 3
      IPHI2         = IPH + 3
      IF(ITHE1.LT.1)    ITHE1 =    1
      IF(ITHE2.GT.NTHE) ITHE2 = NTHE
      IF(IPHI1.LT.1)    IPHI1 =    1
      IF(IPHI2.GT.NPHI) IPHI2 = NPHI
C
      DO ITHE = ITHE1,ITHE2
        DO IPHI = IPHI1,IPHI2
          IF(MASK(IPHI,ITHE,1).GE.1) THEN            ! IS THIS TEST NECESSARY?
            NVIEW_THISIMG = NVIEW_THISIMG + 1
          ENDIF
        ENDDO
      ENDDO

CTSB      USE NVIEW_THISIMG = (ITHE2-ITHE1+1)*(IPHI2-IPHI1+1)      ???

      FACTOR = 1.0/FLOAT(NVIEW_THISIMG)            ! NORMALIZATION FACTOR
C
      DO ITHE = ITHE1,ITHE2
        DO IPHI = IPHI1,IPHI2
          IF(MASK(IPHI,ITHE,1).GE.1) THEN           ! IS THIS TEST NECESSARY?
            MODEL_ID = MASK(IPHI,ITHE,2)
            PTS      = 0.0
            DO IR = ANN_LO,ANN_HI
              R      = MIN(NANNULI, IR+1)
              TWOPIR = 2.0*PI*FLOAT(IR)
              DO JN = JCUT,(NROT_BY_4 - 1)
                IF(JN.GT.0) THEN                         ! TRAP FOR Jo
                   PIXRES = TWOPIR/FLOAT(JN)
                 ELSE 
                   PIXRES = 1000.0
                ENDIF
                IF(PIXRES.LT.PIXRES_MAX .OR. PIXRES.GT.PIXRES_MIN) 
     1           GO TO 200
                J    = JN + 1
                PTS  = PTS + 1.0
                  TMP2 = PFTS(J,R,MODEL_ID)
                  IF(VAROPT) THEN
                  AVER(J,R) = AVER(J,R) + TMP2
                    VAR(J,R)  = 0
                  ENDIF
  200              ENDDO
            ENDDO
          ENDIF
        ENDDO                              ! DO IPHI = IPHI1,IPHI2
      ENDDO                              ! DO ITHE = ITHE1,ITHE2
C
        IF(.NOT.VAROPT) GO TO 500
C  ******************************
C  *    HERE IF SIGCUT > 0.0    *

C      ***NOTE***  THIS NEEDS FIXING BECAUSE LOOP IS NOT CORRECT; ALSO ID NOT
C      RIGHT

C  ******************************
        AVE2 = 0.0
      DO IR = ANN_LO,ANN_HI
        R      = MIN(NANNULI, IR+1)
        TWOPIR = 2.0*PI*FLOAT(IR)
        DO JN = JCUT,(NROT_BY_4 - 1)
          IF(JN.GT.0) THEN                              ! TRAP FOR Jo
             PIXRES = TWOPIR/FLOAT(JN)
           ELSE
             PIXRES = 1000.0
          ENDIF
          IF(PIXRES.LT.PIXRES_MAX .OR. PIXRES.GT.PIXRES_MIN) GO TO 300
          J        = JN + 1
            RMS(J,R) = 0.0
          AVE      = AVER(J,R)*FACTOR
            DO I = 1,NVIEW_THISIMG
              TMP2     = PFTS(J,R,I)
              RMS(J,R) = RMS(J,R) + (TMP2-AVE)**2
          ENDDO
            RMS(J,R) = SQRT(RMS(J,R)*FACTOR)
            AVE2     = AVE2 + RMS(J,R)
  300        ENDDO
      ENDDO
        NUM  = 0
        AVE2 = AVE2/PTS
      DO IR = ANN_LO,ANN_HI
        R      = MIN(NANNULI, IR + 1)
        TWOPIR = 2.0*PI*FLOAT(IR)
        DO JN = JCUT,(NROT_BY_4 - 1)
          IF(JN.GT.0) THEN                        ! TRAP FOR Jo
             PIXRES = TWOPIR/FLOAT(JN)
           ELSE
             PIXRES = 1000.0
          ENDIF
          IF(PIXRES.LT.PIXRES_MAX .OR. PIXRES.GT.PIXRES_MIN) GO TO 400
          J = JN + 1
            IF (RMS(J,R).GT.SIGCUT*AVE2) THEN
              VAR(J,R) = 1
              NUM      = NUM + 1 
            ENDIF
  400        ENDDO
      ENDDO
      PRINT *,'VARIANCE:'
      PRINT *,'USING POINTS > ',SIGCUT,'*',AVE2
      PRINT *,'POINTS FROM PFFT BEING USED: ',NUM
C
  500      NUM = 0
      DO ITHE = ITHE1,ITHE2
        DO IPHI = IPHI1,IPHI2
          IF(MASK(IPHI,ITHE,1).GE.1) THEN            ! IS THIS TEST NECESSARY?
            NUM           = NUM + 1
            MODEL_ID      = MASK(IPHI,ITHE,2)
            PTS           = 0.0
            PFT_AVGS(NUM) = 0.0
              SQ2(NUM)      = 0.0
            DO IR = ANN_LO,ANN_HI
              R      = MIN(NANNULI, IR+1)
              TWOPIR = 2.0*PI*FLOAT(IR)
              DO JN = JCUT,(NROT_BY_4 - 1)
                IF(JN.GT.0) THEN                        ! TRAP FOR Jo
                   PIXRES = TWOPIR/FLOAT(JN)
                 ELSE
                   PIXRES = 1000.0
                ENDIF
                IF(PIXRES.LT.PIXRES_MAX .OR. PIXRES.GT.PIXRES_MIN) 
     1           GO TO 600
                J = JN + 1
                  IF((VAROPT.AND.(VAR(J,R).ne.0)) .OR. .NOT.VAROPT) THEN
                  PTS           = PTS + 1.0
                    TMP2          = PFTS(J,R,MODEL_ID)
                  PFT_AVGS(NUM) = PFT_AVGS(NUM) + TMP2
                  SQ2(NUM)      = SQ2(NUM) + TMP2**2
                  ENDIF
  600              ENDDO
            ENDDO
            PFT_AVGS(NUM) = PFT_AVGS(NUM)/PTS
          ENDIF
        ENDDO                              ! DO IPHI = IPHI1,IPHI2
      ENDDO                              ! DO ITHE = ITHE1,ITHE2
C
      RETURN
      END





      SUBROUTINE PFTCC_RES (FFT1, FFT3, FFT4, FFT5, IDIM1, IDIM2, CC,
     1 DEL_PIXRES)
C
      IMPLICIT NONE
C
      INTEGER*4 IDIM1, IDIM2, IX, IY, XX, BIN, IXTEST
      INTEGER*4 I, J, IX2, IY2
      REAL*4    CC(2,20), RES, YSQ
      REAL*4    FFT1(2*IDIM1,IDIM2), FFT3(2*IDIM1,IDIM2)
      REAL*4    FFT4(2*IDIM1,IDIM2), FFT5(2*IDIM1,IDIM2)
      REAL*4    DEL_PIXRES, SUM13(20), SUM45(20), PTS(20)
      REAL*4    AVG1(20), AVG3(20), AVG4(20), AVG5(20)
      REAL*4    SQ1(20), SQ3(20), SQ4(20), SQ5(20)
      REAL*4    DIV1, DIV3, DIV4, DIV5, AMP1, AMP3, AMP4, AMP5
C  ----------------------------------------------------------------------------
C
      IXTEST  = IDIM1/2
C
C  ***************************
C  *    INITIALIZE ARRAYS    *
C  ***************************
      DO I = 1,20
        DO J = 1,2
          CC(J,I)    = 0.0
        ENDDO
        PTS(I)   = 0.0
        SUM13(I) = 0.0
        SUM45(I) = 0.0
        AVG1(I)  = 0.0
        AVG3(I)  = 0.0
        AVG4(I)  = 0.0
        AVG5(I)  = 0.0
        SQ1(I)   = 0.0
        SQ3(I)   = 0.0
        SQ4(I)   = 0.0
        SQ5(I)   = 0.0
      ENDDO
C
      IY2 = (IDIM2/2) + 1
      DO IY = 1,IY2
        YSQ = FLOAT(IY-1)**2
        IX2 = IDIM1
        IF(IY.EQ.1 .OR. IY.EQ.IY2) THEN
          IX2 = IDIM1/2
        ENDIF
        DO IX = 1,IX2
          IF(IY.EQ.1 .AND. IX.EQ.1) GO TO 100
          XX = (IX-1)*2 + 1
          IF(IX.GT.IXTEST) THEN
             RES = SQRT ( YSQ + FLOAT(IDIM1-IX+1)**2 ) 
           ELSE
             RES = SQRT ( YSQ + FLOAT(IX-1)**2 )
          ENDIF
          AMP1 = SQRT(FFT1(XX,IY)**2 + FFT1(XX+1,IY)**2)
          AMP3 = SQRT(FFT3(XX,IY)**2 + FFT3(XX+1,IY)**2)
          AMP4 = SQRT(FFT4(XX,IY)**2 + FFT4(XX+1,IY)**2)
          AMP5 = SQRT(FFT5(XX,IY)**2 + FFT5(XX+1,IY)**2)
          BIN        = INT( RES/(IDIM1*DEL_PIXRES) ) + 1
          IF(BIN.GT.20) GO TO 100
          PTS(BIN)   = PTS(BIN) + 1.0
          SUM13(BIN) = SUM13(BIN) + AMP1*AMP3
          SUM45(BIN) = SUM45(BIN) + AMP4*AMP5
          AVG1(BIN)  = AVG1(BIN) + AMP1
          AVG3(BIN)  = AVG3(BIN) + AMP3
          AVG4(BIN)  = AVG4(BIN) + AMP4
          AVG5(BIN)  = AVG5(BIN) + AMP5
          SQ1(BIN)   = SQ1(BIN) + AMP1**2
          SQ3(BIN)   = SQ3(BIN) + AMP3**2
          SQ4(BIN)   = SQ4(BIN) + AMP4**2
          SQ5(BIN)   = SQ5(BIN) + AMP5**2
  100        ENDDO
      ENDDO
C
      DO I = 1,20
        IF(PTS(I).LE.0.0) THEN
           AVG1(I) = 0.0
           AVG3(I) = 0.0
           AVG4(I) = 0.0
           AVG5(I) = 0.0
         ELSE
           AVG1(I) = AVG1(I)/PTS(I)
           AVG3(I) = AVG3(I)/PTS(I)
           AVG4(I) = AVG4(I)/PTS(I)
           AVG5(I) = AVG5(I)/PTS(I)
        ENDIF
        DIV1 = SQ1(I) - PTS(I)*(AVG1(I)**2)
        DIV3 = SQ3(I) - PTS(I)*(AVG3(I)**2)
        DIV4 = SQ4(I) - PTS(I)*(AVG4(I)**2)
        DIV5 = SQ5(I) - PTS(I)*(AVG5(I)**2)
        DIV1 = sign(SQRT(abs(DIV1)), DIV1)
        DIV3 = sign(SQRT(abs(DIV3)), DIV3)
        DIV4 = sign(SQRT(abs(DIV4)), DIV4)
        DIV5 = sign(SQRT(abs(DIV5)), DIV5)
        IF(DIV1*DIV3.LE.0.0) THEN
           CC(1,I) = 0.0
         ELSE
           CC(1,I) = (SUM13(I)-AVG1(I)*AVG3(I)*PTS(I))/(DIV1*DIV3)
        ENDIF
        IF(DIV4*DIV5.LE.0.0) THEN
           CC(2,I) = 0.0
         ELSE
           CC(2,I) = (SUM45(I)-AVG4(I)*AVG5(I)*PTS(I))/(DIV4*DIV5)
        ENDIF
      ENDDO
C
      RETURN
      END




